
#include <iostream>

#include "TStyle.h"
#include "TH1.h"
#include "TCanvas.h"

#include "TArtAnaLoop.hh"

#include "TArtCalibNEBULA.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtNEBULAParameters.hh"

static TArtCalibNEBULA *mynebula;
static TArtNEBULAParameters *NEBULAParameters;

static TH1 *hadc, *htdc;

class TArtAnaLoopNEBULA : public TArtAnaLoop
{
public:
  TArtAnaLoopNEBULA() : TArtAnaLoop(){;}
  ~TArtAnaLoopNEBULA(){;}

protected:
  void Construct();
  void Calculate();
  void SuspendAction();
  void Destruct();
};

void TestNEBULA2(char* filename = "ridf/test.ridf", int eventnumber = -1)
{
  static TArtAnaLoopNEBULA analoop;
  analoop.AnaLoop(filename, eventnumber);
}

void TArtAnaLoopNEBULA::Construct()
{
  mynebula= new TArtCalibNEBULA;
  NEBULAParameters = new TArtNEBULAParameters;
  NEBULAParameters->LoadParameter("db/NEBULA.xml");

  hadc = new TH1I("hadc", "hist of adc", 4100, 0, 4100);
  htdc = new TH1I("htdc", "hist of tdc", 4100, 0, 4100);
}

void TArtAnaLoopNEBULA::Calculate()
{
  mynebula->BeginEvent();
  mynebula->ReconstructData();

  Int_t NumNEBULAPla = mynebula->GetNumNEBULAPla();
  for(int i=0; i < NumNEBULAPla; ++i){
    TArtNEBULAPla* pla = mynebula->GetNEBULAPla(i);

    unsigned int adc = pla->GetRawADC();
    unsigned int tdc = pla->GetRawTDC();

    hadc->Fill(adc);
    htdc->Fill(tdc);

    const TArtNEBULAPlaPara* para = mynebula->GetNEBULAPlaPara(i);
    //      std::cout << *para << std::endl;
  }
  mynebula->EndEvent();
  estore->ClearData();
}

void TArtAnaLoopNEBULA::SuspendAction()
{
  static TCanvas* canvas = TCanvas::MakeDefCanvas();
  canvas->Clear();
  canvas->Divide(1,2);
  canvas->cd(1);
  hadc->Draw();
  canvas->cd(2);
  htdc->Draw();
  canvas->RaiseWindow();
  canvas->Update();
}

void TArtAnaLoopNEBULA::Destruct()
{
  delete mynebula;
  delete NEBULAParameters;

  return;
}
