
#include <iostream>
#include <limits>

#include "TThread.h"
#include "TObject.h"
#include "TDirectory.h"

#include "TArtEventStore.h"
#include "TArtStoreManager.h"

#include "TArtAnaLoop.hh"

//_______________________________________________________________________________________

void book(TArtAnaLoop* analoop, TString filename, int eventnumber, TDirectory* directory)
{TArtAnaLoopManager::Book(analoop, filename, eventnumber, directory);}
void start(){TArtAnaLoopManager::Start();}
void stop(){TArtAnaLoopManager::Stop();}
void end(){TArtAnaLoopManager::End();}
void anasta(){TArtAnaLoopManager::AnaSta();}

//_______________________________________________________________________________________

TArtAnaLoopManager::EAnalysisState TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::INIT;
TThread* TArtAnaLoopManager::fThread = 0;

TThread* TArtAnaLoopManager::Book(TArtAnaLoop* analoop, TString filename,
				  int eventnumber, TDirectory* directory)
{
  if(fThread != 0){
    delete fThread;
    fThread = 0;
  }

  TArtAnaLoopStatic::SetAnaLoop(analoop);
  TArtAnaLoopStatic::SetFileName(filename);
  TArtAnaLoopStatic::SetEventNumber(eventnumber);
  TArtAnaLoopStatic::SetDirectory(directory);

  void *(*fpt)(void*) = TArtAnaLoopStatic::AnaLoopStatic;

  return fThread = new TThread((void *(*)(void*))fpt, (void*)0);
}

void TArtAnaLoopManager::Start()
{
  if(kAnalysisState == INIT){
    std::cout << "\n start analoop\n" << std::endl;    
    fThread->Run();
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n ...running\n" << std::endl;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n resume analoop\n" << std::endl;    
    kAnalysisState = RUNNING;
    fThread->Run();
  }else if(kAnalysisState == END){
    std::cout << "\n start analoop\n" << std::endl;
    kAnalysisState = INIT;
    fThread->Run();
  }
}

void TArtAnaLoopManager::Stop()
{
  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n stop analoop\n" << std::endl;
    kAnalysisState = SUSPENDED;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n ...suspended\n" << std::endl;    
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }
}

void TArtAnaLoopManager::End()
{
  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Run();
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }
}

void TArtAnaLoopManager::AnaSta()
{
  std::cout << std::endl;

  TString state;
  switch (kAnalysisState){
  case INIT:
    state = "INIT"; break;
  case RUNNING:
    state = "RUNNING"; break;
  case SUSPENDED:
    state = "SUSPENDED"; break;
  case END:
    state = "END"; break;
  }
  std::cout << " Analysis State: " << state << std::endl;
  std::cout << " File Name: " << TArtAnaLoopStatic::GetFileName() << std::endl;
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  TString classname("");
  int eventnumber(0);
  if(analoop != 0){
    classname = analoop->ClassName();
    eventnumber = analoop->GetEventNumber();
  }
  std::cout << " Class Name: " << classname << std::endl;
  std::cout << " Event Number: " << eventnumber << std::endl;

  std::cout << std::endl;
}

TArtAnaLoopManager::~TArtAnaLoopManager()
{
  delete fThread;
}

//_______________________________________________________________________________________

void* TArtAnaLoopStatic::AnaLoopStatic(void *)
{
  fDirectory->cd();
  fAnaLoop->AnaLoop(fFileName.Data(), fEventNumber);
    
  return 0;
}

void TArtAnaLoopStatic::SetAnaLoop(TArtAnaLoop* analoop){fAnaLoop = analoop;}
void TArtAnaLoopStatic::SetFileName(TString filename){fFileName = filename;}
void TArtAnaLoopStatic::SetEventNumber(int eventnumber){fEventNumber = eventnumber;}
void TArtAnaLoopStatic::SetDirectory(TDirectory* directory)
{
  if(directory == 0) directory = gDirectory;
  fDirectory = directory;
}  
TArtAnaLoop* TArtAnaLoopStatic::GetAnaLoop(){return fAnaLoop;}
TString TArtAnaLoopStatic::GetFileName(){return fFileName;}
int TArtAnaLoopStatic::GetEventNumber(){return fEventNumber;}
TDirectory* TArtAnaLoopStatic::GetDirectory(){return fDirectory;}

TArtAnaLoop* TArtAnaLoopStatic::fAnaLoop = 0;
TString TArtAnaLoopStatic::fFileName = "";
int TArtAnaLoopStatic::fEventNumber = -1;
TDirectory* TArtAnaLoopStatic::fDirectory = 0;

//_______________________________________________________________________________________

TArtAnaLoop::TArtAnaLoop()
  : estore(0), neve(0)
{;}

TArtAnaLoop::~TArtAnaLoop()
{
  delete estore;
  estore = 0;
}

void TArtAnaLoop::AnaLoop(const char* filename, int eventnumber)
{
  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::INIT){
    neve = 0;

    estore = new TArtEventStore;
    estore->Open(filename);

    Construct();

    TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::RUNNING;
  }

  if(estore == 0) return;

  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::RUNNING){
    while(1){
      ++neve;
      if(!estore->GetNextEvent() || (!(eventnumber<0) && neve>eventnumber)){
	std::cout << "\n\n analysis completed!! \n" << std::endl;
	TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::END;
	break;
      }else if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
	break;
      }

      Calculate();
      estore->ClearData();

      if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::SUSPENDED){
	SuspendAction();
	break;
      }
    }
  }

  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
    Destruct();
    delete estore;
    estore = 0;
    delete TArtStoreManager::Instance();
  }
}

